@@ -79,7 +79,7 @@ const EditView = new Lang.Class({
         this._viewAction.enabled = false;
         this._viewAction.connect('activate', Lang.bind(this,
             function() {
-                Application.modeController.setWindowMode(WindowMode.WindowMode.PREVIEW);
+                Application.modeController.goBack();
             }));
 
         Application.documentManager.connect('load-started',
@@ -170,6 +170,7 @@ const EditToolbar = new Lang.Class({
         backButton.connect('clicked', Lang.bind(this,
             function() {
                 Application.documentManager.setActiveItem(null);
+                Application.modeController.goBack(2);
             }));
 
         let viewButton = new Gtk.Button({ label: _("View"),
