@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2013 Red Hat, Inc.
+ * Copyright (c) 2011, 2013, 2015 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -78,7 +78,6 @@ const Embed = new Lang.Class({
     Name: 'Embed',
 
     _init: function() {
-        this._queryErrorId = 0;
         this._noResultsChangeId = 0;
         this._loadShowId = 0;
 
@@ -186,13 +185,16 @@ const Embed = new Lang.Class({
 
     _onActivateResult: function() {
         let windowMode = Application.modeController.getWindowMode();
-        let view = this._getViewFromMode(mode);
+        let view = this._getViewFromMode(windowMode);
         view.activateResult();
     },
 
     _restoreLastPage: function() {
-        let page;
         let windowMode = Application.modeController.getWindowMode();
+        if (windowMode == WindowMode.WindowMode.NONE)
+            return;
+
+        let page;
 
         switch (windowMode) {
         case WindowMode.WindowMode.COLLECTIONS:
