@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Red Hat, Inc.
+ * Copyright (c) 2011, 2015 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@ const Signals = imports.signals;
 const Tweener = imports.tweener.tweener;
 
 const Application = imports.application;
+const ErrorBox = imports.errorBox;
 const MainToolbar = imports.mainToolbar;
 const Places = imports.places;
 const Searchbar = imports.searchbar;
@@ -59,27 +60,33 @@ const PreviewView = new Lang.Class({
         this._fsToolbar = null;
         this._overlay = overlay;
         this._lastSearch = '';
-        this._loadError = false;
 
         Application.modeController.connect('fullscreen-changed', Lang.bind(this,
             this._onFullscreenChanged));
         Application.modeController.connect('window-mode-changed', Lang.bind(this,
             this._onWindowModeChanged));
 
-        this.widget = new Gtk.ScrolledWindow({ hexpand: true,
-                                               vexpand: true });
-        this.widget.get_style_context().add_class('documents-scrolledwin');
-        this.widget.get_hscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
-        this.widget.get_vscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
-        this.widget.get_hadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
-        this.widget.get_vadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
+        this.widget = new Gtk.Stack({ homogeneous: true,
+                                      transition_type: Gtk.StackTransitionType.CROSSFADE });
+
+        this._errorBox = new ErrorBox.ErrorBox();
+        this.widget.add_named(this._errorBox.widget, 'error');
+
+        this._sw = new Gtk.ScrolledWindow({ hexpand: true,
+                                            vexpand: true });
+        this._sw.get_style_context().add_class('documents-scrolledwin');
+        this._sw.get_hscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
+        this._sw.get_vscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
+        this._sw.get_hadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
+        this._sw.get_vadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
+        this.widget.add_named(this._sw, 'view');
 
         this._createView();
 
         // create context menu
         let model = this._getPreviewContextMenu();
         this._previewContextMenu = Gtk.Menu.new_from_model(model);
-        this._previewContextMenu.attach_to_widget(this.widget, null);
+        this._previewContextMenu.attach_to_widget(this._sw, null);
 
         this.widget.show_all();
 
@@ -138,8 +145,6 @@ const PreviewView = new Lang.Class({
 
         Application.documentManager.connect('load-started',
                                             Lang.bind(this, this._onLoadStarted));
-        Application.documentManager.connect('load-finished',
-                                            Lang.bind(this, this._onLoadFinished));
         Application.documentManager.connect('load-error',
                                             Lang.bind(this, this._onLoadError));
     },
@@ -150,24 +155,10 @@ const PreviewView = new Lang.Class({
         this._copy.enabled = false;
     },
 
-    _onLoadFinished: function(manager, doc, docModel) {
-        this._showPlaces.enabled = true;
-        this._togglePresentation.enabled = true;
-        this._navControls.show();
-
-        if (!Application.documentManager.metadata)
-            return;
-
-        this._bookmarks = new GdPrivate.Bookmarks({ metadata: Application.documentManager.metadata });
-        this._bookmarkPage.enabled = true;
-    },
-
     _onLoadError: function(manager, doc, message, exception) {
         this._controlsVisible = true;
-
-        this._loadError = true;
         this._syncControlsVisible();
-        this._loadError = false;
+        this._setError(message, exception.message);
     },
 
     _onActionStateChanged: function(action, state) {
@@ -206,6 +197,11 @@ const PreviewView = new Lang.Class({
         this._bookmarkPage.state = GLib.Variant.new('b', hasBookmark);
     },
 
+    _setError: function(primary, secondary) {
+        this._errorBox.update(primary, secondary);
+        this.widget.set_visible_child_name('error');
+    },
+
     _showPlaces: function() {
         let dialog = new Places.PlacesDialog(this._model, this._bookmarks);
         dialog.widget.connect('response', Lang.bind(this,
@@ -330,7 +326,7 @@ const PreviewView = new Lang.Class({
 
     _createView: function() {
         this.view = EvView.View.new();
-        this.widget.add(this.view);
+        this._sw.add(this.view);
         this.view.show();
 
         this.view.connect('notify::can-zoom-in', Lang.bind(this,
@@ -347,6 +343,7 @@ const PreviewView = new Lang.Class({
             this._handleExternalLink));
 
         this._navControls = new PreviewNavControls(this, this._overlay);
+        this.widget.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
     },
 
     _getPreviewContextMenu: function() {
@@ -537,6 +534,11 @@ const PreviewView = new Lang.Class({
         if (this._model) {
             this.view.set_model(this._model);
             this._navControls.setModel(model);
+            this._navControls.show();
+            this._togglePresentation.enabled = true;
+
+            if (Application.documentManager.metadata)
+                this._bookmarks = new GdPrivate.Bookmarks({ metadata: Application.documentManager.metadata });
 
             let hasMultiplePages = (this._model.document.get_n_pages() > 1);
             this._bookmarkPage.enabled = hasMultiplePages && this._bookmarks;
@@ -572,10 +574,10 @@ const PreviewNavControls = new Lang.Class({
         this._overlay = overlay;
 
         this._visible = false;
+        this._visibleInternal = false;
         this._pageChangedId = 0;
         this._autoHideId = 0;
         this._motionId = 0;
-        this._hover = false;
 
         this.bar_widget = new GdPrivate.NavBar({ document_model: this._model,
                                                  margin: _PREVIEW_NAVBAR_MARGIN,
@@ -634,35 +636,56 @@ const PreviewNavControls = new Lang.Class({
 
         this._overlay.connect('motion-notify-event', Lang.bind(this, this._onMotion));
 
+        this._tapGesture = new Gtk.GestureMultiPress({ propagation_phase: Gtk.PropagationPhase.CAPTURE,
+                                                       touch_only: true,
+                                                       widget: this._previewView.view });
+        this._tapGesture.connect('released', Lang.bind(this, this._onMultiPressReleased));
+        this._tapGesture.connect('stopped', Lang.bind(this, this._onMultiPressStopped));
     },
 
     _onEnterNotify: function() {
-        this._hover = true;
         this._unqueueAutoHide();
         return false;
     },
 
     _onLeaveNotify: function() {
-        this._hover = false;
         this._queueAutoHide();
         return false;
     },
 
     _motionTimeout: function() {
         this._motionId = 0;
+        this._visibleInternal = true;
         this._updateVisibility();
+        this._queueAutoHide();
         return false;
     },
 
-    _onMotion: function() {
+    _onMotion: function(widget, event) {
         if (this._motionId != 0) {
             return false;
         }
 
+        let device = event.get_source_device();
+        if (device.input_source == Gdk.InputSource.TOUCHSCREEN) {
+            return false;
+        }
+
         this._motionId = Mainloop.idle_add(Lang.bind(this, this._motionTimeout));
         return false;
     },
 
+    _onMultiPressReleased: function() {
+        this._tapGesture.set_state(Gtk.EventSequenceState.CLAIMED);
+        this._visibleInternal = !this._visibleInternal;
+        this._unqueueAutoHide();
+        this._updateVisibility();
+    },
+
+    _onMultiPressStopped: function() {
+        this._tapGesture.set_state(Gtk.EventSequenceState.DENIED);
+    },
+
     _onPrevClicked: function() {
         this._previewView.view.previous_page();
     },
@@ -672,10 +695,9 @@ const PreviewNavControls = new Lang.Class({
     },
 
     _autoHide: function() {
-        this._fadeOutButton(this.bar_widget);
-        this._fadeOutButton(this.prev_widget);
-        this._fadeOutButton(this.next_widget);
         this._autoHideId = 0;
+        this._visibleInternal = false;
+        this._updateVisibility();
         return false;
     },
 
@@ -693,7 +715,7 @@ const PreviewNavControls = new Lang.Class({
     },
 
     _updateVisibility: function() {
-        if (!this._model || !this._visible) {
+        if (!this._model || !this._visible || !this._visibleInternal) {
             this._fadeOutButton(this.bar_widget);
             this._fadeOutButton(this.prev_widget);
             this._fadeOutButton(this.next_widget);
@@ -712,9 +734,6 @@ const PreviewNavControls = new Lang.Class({
             this._fadeInButton(this.next_widget);
         else
             this._fadeOutButton(this.next_widget);
-
-        if (!this._hover)
-            this._queueAutoHide();
     },
 
     setModel: function(model) {
@@ -728,8 +747,6 @@ const PreviewNavControls = new Lang.Class({
 
         if (this._model)
             this._pageChangedId = this._model.connect('page-changed', Lang.bind(this, this._updateVisibility));
-
-        this._updateVisibility();
     },
 
     _fadeInButton: function(widget) {
@@ -753,11 +770,14 @@ const PreviewNavControls = new Lang.Class({
 
     show: function() {
         this._visible = true;
+        this._visibleInternal = true;
         this._updateVisibility();
+        this._queueAutoHide();
     },
 
     hide: function() {
         this._visible = false;
+        this._visibleInternal = false;
         this._updateVisibility();
     },
 
@@ -765,6 +785,7 @@ const PreviewNavControls = new Lang.Class({
         this.bar_widget.destroy();
         this.prev_widget.destroy();
         this.next_widget.destroy();
+        this._tapGesture = null;
     }
 });
 
